@@ -1,4 +1,4 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 #
 # Electrum - lightweight Bitcoin client
 # Copyright (C) 2015 kyuupichan@gmail
@@ -22,13 +22,18 @@
 # ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 # CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import print_function
+from __future__ import unicode_literals
 
 from collections import defaultdict, namedtuple
 from math import floor, log10
 
-from bitcoin import sha256, COIN, TYPE_ADDRESS
-from transaction import Transaction
-from util import NotEnoughFunds, PrintError, profiler
+from .bitcoin import sha256, COIN, TYPE_ADDRESS
+from .transaction import Transaction
+from .util import NotEnoughFunds, PrintError, profiler
+
 
 # A simple deterministic PRNG.  Used to deterministically shuffle a
 # set of coins - the same set of coins should produce the same output.
@@ -36,7 +41,6 @@ from util import NotEnoughFunds, PrintError, profiler
 # so if sending twice from the same UTXO set we choose the same UTXOs
 # to spend.  This prevents attacks on users by malicious or stale
 # servers.
-
 class PRNG:
     def __init__(self, seed):
         self.sha = sha256(seed)
@@ -63,7 +67,7 @@ class PRNG:
         return seq[self.randint(0, len(seq))]
 
     def shuffle(self, x):
-        for i in reversed(xrange(1, len(x))):
+        for i in reversed(range(1, len(x))):
             # pick an element in x[:i+1] with which to exchange x[i]
             j = self.randint(0, i+1)
             x[i], x[j] = x[j], x[i]
@@ -97,7 +101,7 @@ class CoinChooserBase(PrintError):
             value = sum(coin['value'] for coin in coins)
             return Bucket(desc, size, value, coins)
 
-        return map(make_Bucket, buckets.keys(), buckets.values())
+        return list(map(make_Bucket, buckets.keys(), buckets.values()))
 
     def penalty_func(self, tx):
         def penalty(candidate):
@@ -123,7 +127,7 @@ class CoinChooserBase(PrintError):
             s = str(val)
             return len(s) - len(s.rstrip('0'))
 
-        zeroes = map(trailing_zeroes, output_amounts)
+        zeroes = [trailing_zeroes(i) for i in output_amounts]
         min_zeroes = min(zeroes)
         max_zeroes = max(zeroes)
         zeroes = range(max(0, min_zeroes - 1), (max_zeroes + 1) + 1)
@@ -132,7 +136,7 @@ class CoinChooserBase(PrintError):
         remaining = change_amount
         amounts = []
         while n > 1:
-            average = remaining // n
+            average = remaining / n
             amount = self.p.randint(int(average * 0.7), int(average * 1.3))
             precision = min(self.p.choice(zeroes), int(floor(log10(amount))))
             amount = int(round(amount, -precision))
@@ -209,6 +213,9 @@ class CoinChooserBase(PrintError):
 
         return tx
 
+    def choose_buckets(self, buckets, sufficient_funds, penalty_func):
+        raise NotImplemented('To be subclassed')
+
 class CoinChooserOldestFirst(CoinChooserBase):
     '''Maximize transaction priority. Select the oldest unspent
     transaction outputs in your wallet, that are sufficient to cover
@@ -247,7 +254,7 @@ class CoinChooserRandom(CoinChooserBase):
 
         # And now some random ones
         attempts = min(100, (len(buckets) - 1) * 10 + 1)
-        permutation = range(len(buckets))
+        permutation = list(range(len(buckets)))
         for i in range(attempts):
             # Get a random permutation of the buckets, and
             # incrementally combine buckets until sufficient
