@@ -1,4 +1,4 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 #
 # Electrum - lightweight Bitcoin client
 # Copyright (C) 2011 thomasv@gitorious
@@ -22,6 +22,10 @@
 # ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 # CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import print_function
+from __future__ import unicode_literals
 
 import os
 import sys
@@ -35,15 +39,16 @@ import base64
 from functools import wraps
 from decimal import Decimal
 
-import util
-from util import print_msg, format_satoshis, print_stderr
-import bitcoin
-from bitcoin import is_address, hash_160, COIN, TYPE_ADDRESS
-import transaction
-from transaction import Transaction
-import paymentrequest
-from paymentrequest import PR_PAID, PR_UNPAID, PR_UNKNOWN, PR_EXPIRED
-import contacts
+from .import util
+from .util import print_msg, format_satoshis, print_stderr
+from .import bitcoin
+from .bitcoin import is_address,  hash_160, COIN, TYPE_ADDRESS
+from .transaction import Transaction
+from .import paymentrequest
+from .paymentrequest import PR_PAID, PR_UNPAID, PR_UNKNOWN, PR_EXPIRED
+from .import contacts
+from .plugins import run_hook
+
 known_commands = {}
 
 
@@ -53,7 +58,6 @@ def satoshis(amount):
 
 
 class Command:
-
     def __init__(self, func, s):
         self.name = func.__name__
         self.requires_network = 'n' in s
@@ -61,8 +65,8 @@ class Command:
         self.requires_password = 'p' in s
         self.description = func.__doc__
         self.help = self.description.split('.')[0] if self.description else None
-        varnames = func.func_code.co_varnames[1:func.func_code.co_argcount]
-        self.defaults = func.func_defaults
+        varnames = func.__code__.co_varnames[1:func.__code__.co_argcount]
+        self.defaults = func.__defaults__
         if self.defaults:
             n = len(self.defaults)
             self.params = list(varnames[:-n])
@@ -80,6 +84,13 @@ def command(s):
         known_commands[name] = Command(func, s)
         @wraps(func)
         def func_wrapper(*args, **kwargs):
+            c = known_commands[func.__name__]
+            wallet = args[0].wallet
+            password = kwargs.get('password')
+            if c.requires_wallet and wallet is None:
+                raise BaseException("wallet not loaded. Use 'electrum daemon load_wallet'")
+            if c.requires_password and password is None and wallet.storage.get('use_encryption'):
+                return {'error': 'Password required' }
             return func(*args, **kwargs)
         return func_wrapper
     return decorator
@@ -119,7 +130,7 @@ class Commands:
         return ' '.join(sorted(known_commands.keys()))
 
     @command('')
-    def create(self):
+    def create(self, segwit=False):
         """Create a new wallet"""
         raise BaseException('Not a JSON-RPC command')
 
@@ -134,7 +145,8 @@ class Commands:
     @command('wp')
     def password(self, password=None, new_password=None):
         """Change wallet password. """
-        self.wallet.update_password(password, new_password)
+        b = self.wallet.storage.is_encrypted()
+        self.wallet.update_password(password, new_password, b)
         self.wallet.storage.write()
         return {'password':self.wallet.has_password()}
 
@@ -154,16 +166,17 @@ class Commands:
         return True
 
     @command('')
-    def make_seed(self, nbits=132, entropy=1, language=None):
+    def make_seed(self, nbits=132, entropy=1, language=None, segwit=False):
         """Create a seed"""
-        from mnemonic import Mnemonic
-        s = Mnemonic(language).make_seed('standard', nbits, custom_entropy=entropy)
-        return s.encode('utf8')
+        from .mnemonic import Mnemonic
+        t = 'segwit' if segwit else 'standard'
+        s = Mnemonic(language).make_seed(t, nbits, custom_entropy=entropy)
+        return s
 
     @command('')
     def check_seed(self, seed, entropy=1, language=None):
         """Check that a seed was generated with given entropy"""
-        from mnemonic import Mnemonic
+        from .mnemonic import Mnemonic
         return Mnemonic(language).check_seed(seed, entropy)
 
     @command('n')
@@ -180,7 +193,7 @@ class Commands:
         l = copy.deepcopy(self.wallet.get_utxos(exclude_frozen=False))
         for i in l:
             v = i["value"]
-            i["value"] = float(v)/COIN if v is not None else None
+            i["value"] = str(Decimal(v)/COIN) if v is not None else None
         return l
 
     @command('n')
@@ -190,14 +203,6 @@ class Commands:
         """
         return self.network.synchronous_get(('blockchain.address.listunspent', [address]))
 
-    @command('n')
-    def getutxoaddress(self, txid, pos):
-        """Get the address of a UTXO. Note: This is a walletless server query, results are
-        not checked by SPV.
-        """
-        r = self.network.synchronous_get(('blockchain.utxo.get_address', [txid, pos]))
-        return {'address': r}
-
     @command('')
     def serialize(self, jsontx):
         """Create a transaction from json inputs.
@@ -213,18 +218,17 @@ class Commands:
                 prevout_hash, prevout_n = txin['output'].split(':')
                 txin['prevout_n'] = int(prevout_n)
                 txin['prevout_hash'] = prevout_hash
-            if txin.get('redeemPubkey'):
-                pubkey = txin['redeemPubkey']
-                txin['type'] = 'p2pkh'
+            sec = txin.get('privkey')
+            if sec:
+                txin_type, privkey, compressed = bitcoin.deserialize_privkey(sec)
+                pubkey = bitcoin.public_key_from_private_key(privkey, compressed)
+                keypairs[pubkey] = privkey, compressed
+                txin['type'] = txin_type
                 txin['x_pubkeys'] = [pubkey]
                 txin['signatures'] = [None]
                 txin['num_sig'] = 1
-                if txin.get('privkey'):
-                    keypairs[pubkey] = txin['privkey']
-            elif txin.get('redeemScript'):
-                raise BaseException('Not implemented')
 
-        outputs = map(lambda x: (TYPE_ADDRESS, x['address'], int(x['value'])), outputs)
+        outputs = [(TYPE_ADDRESS, x['address'], int(x['value'])) for x in outputs]
         tx = Transaction.from_io(inputs, outputs, locktime=locktime)
         tx.sign(keypairs)
         return tx.as_dict()
@@ -234,10 +238,11 @@ class Commands:
         """Sign a transaction. The wallet keys will be used unless a private key is provided."""
         tx = Transaction(tx)
         if privkey:
-            pubkey = bitcoin.public_key_from_private_key(privkey)
-            h160 = bitcoin.hash_160(pubkey.decode('hex'))
-            x_pubkey = 'fd' + (chr(0) + h160).encode('hex')
-            tx.sign({x_pubkey:privkey})
+            txin_type, privkey2, compressed = bitcoin.deserialize_privkey(privkey)
+            pubkey = bitcoin.public_key_from_private_key(privkey2, compressed)
+            h160 = bitcoin.hash_160(bfh(pubkey))
+            x_pubkey = 'fd' + bh2u(b'\x00' + h160)
+            tx.sign({x_pubkey:(privkey2, compressed)})
         else:
             self.wallet.sign_transaction(tx, password)
         return tx.as_dict()
@@ -258,8 +263,8 @@ class Commands:
     def createmultisig(self, num, pubkeys):
         """Create multisig address"""
         assert isinstance(pubkeys, list), (type(num), type(pubkeys))
-        redeem_script = transaction.multisig_script(pubkeys, num)
-        address = bitcoin.hash160_to_p2sh(hash_160(redeem_script.decode('hex')))
+        redeem_script = Transaction.multisig_script(pubkeys, num)
+        address = bitcoin.hash160_to_p2sh(hash_160(bfh(redeem_script)))
         return {'address':address, 'redeemScript':redeem_script}
 
     @command('w')
@@ -276,9 +281,9 @@ class Commands:
     def getprivatekeys(self, address, password=None):
         """Get private keys of addresses. You may pass a single wallet address, or a list of wallet addresses."""
         if is_address(address):
-            return self.wallet.get_private_key(address, password)
+            return self.wallet.export_private_key(address, password)[0]
         domain = address
-        return [self.wallet.get_private_key(address, password) for address in domain]
+        return [self.wallet.export_private_key(address, password)[0] for address in domain]
 
     @command('w')
     def ismine(self, address):
@@ -344,7 +349,7 @@ class Commands:
     @command('')
     def version(self):
         """Return the version of electrum."""
-        from version import ELECTRUM_VERSION
+        from .version import ELECTRUM_VERSION
         return ELECTRUM_VERSION
 
     @command('w')
@@ -361,15 +366,15 @@ class Commands:
     def getseed(self, password=None):
         """Get seed phrase. Print the generation seed of your wallet."""
         s = self.wallet.get_seed(password)
-        return s.encode('utf8')
+        return s
 
     @command('wp')
     def importprivkey(self, privkey, password=None):
-        """Import a private key. """
+        """Import a private key."""
         if not self.wallet.can_import_privkey():
             return "Error: This type of wallet cannot import private keys. Try to create a new wallet with that key."
         try:
-            addr = self.wallet.import_key(privkey, password)
+            addr = self.wallet.import_private_key(privkey, password)
             out = "Keypair imported: " + addr
         except BaseException as e:
             out = "Error: " + str(e)
@@ -384,12 +389,12 @@ class Commands:
         return out['address']
 
     @command('nw')
-    def sweep(self, privkey, destination, tx_fee=None, nocheck=False, imax=100):
+    def sweep(self, privkey, destination, fee=None, nocheck=False, imax=100):
         """Sweep private keys. Returns a transaction that spends UTXOs from
         privkey to a destination address. The transaction is not
         broadcasted."""
-        tx_fee = satoshis(tx_fee)
-        privkeys = privkey if type(privkey) is list else [privkey]
+        tx_fee = satoshis(fee)
+        privkeys = privkey.split()
         self.nocheck = nocheck
         dest = self._resolver(destination)
         tx = self.wallet.sweep(privkeys, self.network, self.config, dest, tx_fee, imax)
@@ -400,7 +405,7 @@ class Commands:
         """Sign a message with a key. Use quotes if your message contains
         whitespaces"""
         sig = self.wallet.sign_message(address, message, password)
-        return base64.b64encode(sig)
+        return base64.b64encode(sig).decode('ascii')
 
     @command('')
     def verifymessage(self, address, signature, message):
@@ -425,22 +430,23 @@ class Commands:
         if rbf:
             tx.set_rbf(True)
         if not unsigned:
+            run_hook('sign_tx', self.wallet, tx)
             self.wallet.sign_transaction(tx, password)
         return tx
 
     @command('wp')
-    def payto(self, destination, amount, tx_fee=None, from_addr=None, change_addr=None, nocheck=False, unsigned=False, rbf=False, password=None, locktime=None):
+    def payto(self, destination, amount, fee=None, from_addr=None, change_addr=None, nocheck=False, unsigned=False, rbf=False, password=None, locktime=None):
         """Create a transaction. """
-        tx_fee = satoshis(tx_fee)
-        domain = [from_addr] if from_addr else None
+        tx_fee = satoshis(fee)
+        domain = from_addr.split(',') if from_addr else None
         tx = self._mktx([(destination, amount)], tx_fee, change_addr, domain, nocheck, unsigned, rbf, password, locktime)
         return tx.as_dict()
 
     @command('wp')
-    def paytomany(self, outputs, tx_fee=None, from_addr=None, change_addr=None, nocheck=False, unsigned=False, rbf=False, password=None, locktime=None):
+    def paytomany(self, outputs, fee=None, from_addr=None, change_addr=None, nocheck=False, unsigned=False, rbf=False, password=None, locktime=None):
         """Create a multi-output transaction. """
-        tx_fee = satoshis(tx_fee)
-        domain = [from_addr] if from_addr else None
+        tx_fee = satoshis(fee)
+        domain = from_addr.split(',') if from_addr else None
         tx = self._mktx(outputs, tx_fee, change_addr, domain, nocheck, unsigned, rbf, password, locktime)
         return tx.as_dict()
 
@@ -480,7 +486,7 @@ class Commands:
                 'input_addresses': input_addresses,
                 'output_addresses': output_addresses,
                 'label': label,
-                'value': float(value)/COIN if value is not None else None,
+                'value': str(Decimal(value)/COIN) if value is not None else None,
                 'height': height,
                 'confirmations': conf
             })
@@ -512,7 +518,7 @@ class Commands:
         return results
 
     @command('w')
-    def listaddresses(self, receiving=False, change=False, show_labels=False, frozen=False, unused=False, funded=False, show_balance=False):
+    def listaddresses(self, receiving=False, change=False, labels=False, frozen=False, unused=False, funded=False, balance=False):
         """List wallet addresses. Returns the list of all addresses in your wallet. Use optional arguments to filter the results."""
         out = []
         for addr in self.wallet.get_addresses():
@@ -527,10 +533,12 @@ class Commands:
             if funded and self.wallet.is_empty(addr):
                 continue
             item = addr
-            if show_balance:
-                item += ", "+ format_satoshis(sum(self.wallet.get_addr_balance(addr)))
-            if show_labels:
-                item += ', ' + repr(self.wallet.labels.get(addr, ''))
+            if labels or balance:
+                item = (item,)
+            if balance:
+                item += (format_satoshis(sum(self.wallet.get_addr_balance(addr))),)
+            if labels:
+                item += (repr(self.wallet.labels.get(addr, '')),)
             out.append(item)
         return out
 
@@ -594,25 +602,25 @@ class Commands:
         else:
             f = None
         if f is not None:
-            out = filter(lambda x: x.get('status')==f, out)
-        return map(self._format_request, out)
+            out = list(filter(lambda x: x.get('status')==f, out))
+        return list(map(self._format_request, out))
 
     @command('w')
-    def getunusedaddress(self,force=False):
-        """Returns the first unused address."""
-        addr = self.wallet.get_unused_address()
-        if addr is None and force:
-            addr = self.wallet.create_new_address(False)
-
-        if addr:
-            return addr
-        else:
-            return False
+    def createnewaddress(self):
+        """Create a new receiving address, beyond the gap limit of the wallet"""
+        return self.wallet.create_new_address(False)
 
+    @command('w')
+    def getunusedaddress(self):
+        """Returns the first unused address of the wallet, or None if all addresses are used.
+        An address is considered as used if it has received a transaction, or if it is used in a payment request."""
+        return self.wallet.get_unused_address()
 
     @command('w')
     def addrequest(self, amount, memo='', expiration=None, force=False):
-        """Create a payment request."""
+        """Create a payment request, using the first unused address of the wallet.
+        The address will be condidered as used after this operation.
+        If no payment is received, the address will be considered as unused if the payment request is deleted from the wallet."""
         addr = self.wallet.get_unused_address()
         if addr is None:
             if force:
@@ -688,56 +696,57 @@ param_descriptions = {
     'amount': 'Amount to be sent (in BTC). Type \'!\' to send the maximum available.',
     'requested_amount': 'Requested amount (in BTC).',
     'outputs': 'list of ["address", amount]',
+    'redeem_script': 'redeem script (hexadecimal)',
 }
 
 command_options = {
-    'password':    ("-W", "--password",    "Password"),
-    'new_password':(None, "--new_password","New Password"),
-    'receiving':   (None, "--receiving",   "Show only receiving addresses"),
-    'change':      (None, "--change",      "Show only change addresses"),
-    'frozen':      (None, "--frozen",      "Show only frozen addresses"),
-    'unused':      (None, "--unused",      "Show only unused addresses"),
-    'funded':      (None, "--funded",      "Show only funded addresses"),
-    'show_balance':("-b", "--balance",     "Show the balances of listed addresses"),
-    'show_labels': ("-l", "--labels",      "Show the labels of listed addresses"),
-    'nocheck':     (None, "--nocheck",     "Do not verify aliases"),
-    'imax':        (None, "--imax",        "Maximum number of inputs"),
-    'tx_fee':      ("-f", "--fee",         "Transaction fee (in BTC)"),
-    'from_addr':   ("-F", "--from",        "Source address. If it isn't in the wallet, it will ask for the private key unless supplied in the format public_key:private_key. It's not saved in the wallet."),
-    'change_addr': ("-c", "--change",      "Change address. Default is a spare address, or the source address if it's not in the wallet"),
-    'nbits':       (None, "--nbits",       "Number of bits of entropy"),
-    'entropy':     (None, "--entropy",     "Custom entropy"),
-    'language':    ("-L", "--lang",        "Default language for wordlist"),
-    'gap_limit':   ("-G", "--gap",         "Gap limit"),
-    'privkey':     (None, "--privkey",     "Private key. Set to '?' to get a prompt."),
-    'unsigned':    ("-u", "--unsigned",    "Do not sign transaction"),
-    'rbf':         (None, "--rbf",         "Replace-by-fee transaction"),
-    'locktime':    (None, "--locktime",    "Set locktime block number"),
-    'domain':      ("-D", "--domain",      "List of addresses"),
-    'memo':        ("-m", "--memo",        "Description of the request"),
-    'expiration':  (None, "--expiration",  "Time in seconds"),
-    'timeout':     (None, "--timeout",     "Timeout in seconds"),
-    'force':       (None, "--force",       "Create new address beyond gap limit, if no more addresses are available."),
-    'pending':     (None, "--pending",     "Show only pending requests."),
-    'expired':     (None, "--expired",     "Show only expired requests."),
-    'paid':        (None, "--paid",        "Show only paid requests."),
+    'password':    ("-W", "Password"),
+    'new_password':(None, "New Password"),
+    'receiving':   (None, "Show only receiving addresses"),
+    'change':      (None, "Show only change addresses"),
+    'frozen':      (None, "Show only frozen addresses"),
+    'unused':      (None, "Show only unused addresses"),
+    'funded':      (None, "Show only funded addresses"),
+    'balance':     ("-b", "Show the balances of listed addresses"),
+    'labels':      ("-l", "Show the labels of listed addresses"),
+    'nocheck':     (None, "Do not verify aliases"),
+    'imax':        (None, "Maximum number of inputs"),
+    'fee':         ("-f", "Transaction fee (in BTC)"),
+    'from_addr':   ("-F", "Source address. If it isn't in the wallet, it will ask for the private key unless supplied in the format public_key:private_key. It's not saved in the wallet."),
+    'change_addr': ("-c", "Change address. Default is a spare address, or the source address if it's not in the wallet"),
+    'nbits':       (None, "Number of bits of entropy"),
+    'entropy':     (None, "Custom entropy"),
+    'segwit':      (None, "Create segwit seed"),
+    'language':    ("-L", "Default language for wordlist"),
+    'privkey':     (None, "Private key. Set to '?' to get a prompt."),
+    'unsigned':    ("-u", "Do not sign transaction"),
+    'rbf':         (None, "Replace-by-fee transaction"),
+    'locktime':    (None, "Set locktime block number"),
+    'domain':      ("-D", "List of addresses"),
+    'memo':        ("-m", "Description of the request"),
+    'expiration':  (None, "Time in seconds"),
+    'timeout':     (None, "Timeout in seconds"),
+    'force':       (None, "Create new address beyond gap limit, if no more addresses are available."),
+    'pending':     (None, "Show only pending requests."),
+    'expired':     (None, "Show only expired requests."),
+    'paid':        (None, "Show only paid requests."),
 }
 
 
 # don't use floats because of rounding errors
-from transaction import tx_from_str
+from .transaction import tx_from_str
 json_loads = lambda x: json.loads(x, parse_float=lambda x: str(Decimal(x)))
 arg_types = {
     'num': int,
     'nbits': int,
     'imax': int,
-    'entropy': long,
+    'entropy': int,
     'tx': tx_from_str,
     'pubkeys': json_loads,
     'jsontx': json_loads,
     'inputs': json_loads,
     'outputs': json_loads,
-    'tx_fee': lambda x: str(Decimal(x)) if x is not None else None,
+    'fee': lambda x: str(Decimal(x)) if x is not None else None,
     'amount': lambda x: str(Decimal(x)) if x != '!' else '!',
     'locktime': int,
 }
@@ -819,8 +828,6 @@ def add_global_options(parser):
     group.add_argument("-P", "--portable", action="store_true", dest="portable", default=False, help="Use local 'electrum_data' directory")
     group.add_argument("-w", "--wallet", dest="wallet_path", help="wallet path")
     group.add_argument("--testnet", action="store_true", dest="testnet", default=False, help="Use Testnet")
-    group.add_argument("--segwit", action="store_true", dest="segwit", default=False, help="The Wizard will create Segwit seed phrases (Testnet only).")
-    group.add_argument("--nolnet", action="store_true", dest="nolnet", default=False, help="Use Nolnet")
 
 def get_parser():
     # create main parser
@@ -851,7 +858,8 @@ def get_parser():
         if cmdname == 'restore':
             p.add_argument("-o", "--offline", action="store_true", dest="offline", default=False, help="Run offline")
         for optname, default in zip(cmd.options, cmd.defaults):
-            a, b, help = command_options[optname]
+            a, help = command_options[optname]
+            b = '--' + optname
             action = "store_true" if type(default) is bool else 'store'
             args = (a, b) if a else (b,)
             if action == 'store':
