@@ -1,4 +1,4 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 #
 # Electrum - lightweight Bitcoin client
 # Copyright (C) 2011 thomasv@gitorious
@@ -22,6 +22,10 @@
 # ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 # CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import print_function
+from __future__ import unicode_literals
 
 
 # list of words from http://en.wiktionary.org/wiki/Wiktionary:Frequency_lists/Contemporary_poetry
@@ -1665,18 +1669,19 @@ n = 1626
 def mn_encode( message ):
     assert len(message) % 8 == 0
     out = []
-    for i in range(len(message)/8):
+    for i in range(len(message)//8):
         word = message[8*i:8*i+8]
         x = int(word, 16)
         w1 = (x%n)
-        w2 = ((x/n) + w1)%n
-        w3 = ((x/n/n) + w2)%n
+        w2 = ((x//n) + w1)%n
+        w3 = ((x//n//n) + w2)%n
         out += [ words[w1], words[w2], words[w3] ]
     return out
 
+
 def mn_decode( wlist ):
     out = ''
-    for i in range(len(wlist)/3):
+    for i in range(len(wlist)//3):
         word1, word2, word3 = wlist[3*i:3*i+3]
         w1 =  words.index(word1)
         w2 = (words.index(word2))%n
@@ -1688,9 +1693,9 @@ def mn_decode( wlist ):
 
 if __name__ == '__main__':
     import sys
-    if len( sys.argv ) == 1:
-        print 'I need arguments: a hex string to encode, or a list of words to decode'
-    elif len( sys.argv ) == 2:
-        print ' '.join(mn_encode(sys.argv[1]))
+    if len(sys.argv) == 1:
+        print('I need arguments: a hex string to encode, or a list of words to decode')
+    elif len(sys.argv) == 2:
+        print(' '.join(mn_encode(sys.argv[1])))
     else:
-        print mn_decode(sys.argv[1:])
+        print(mn_decode(sys.argv[1:]))
