@@ -1,4 +1,4 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 #
 # Electrum - lightweight Bitcoin client
 # Copyright (C) 2014 Thomas Voegtlin
@@ -22,14 +22,17 @@
 # ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 # CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
-
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import print_function
+from __future__ import unicode_literals
 
 from threading import Lock
 import hashlib
 
-from bitcoin import Hash, hash_encode
-from transaction import Transaction
-from util import print_error, print_msg, ThreadJob
+from .bitcoin import Hash, hash_encode
+from .transaction import Transaction
+from .util import print_error, print_msg, ThreadJob, bh2u
 
 
 class Synchronizer(ThreadJob):
@@ -48,7 +51,7 @@ class Synchronizer(ThreadJob):
         self.network = network
         self.new_addresses = set()
         # Entries are (tx_hash, tx_height) tuples
-        self.requested_tx = set()
+        self.requested_tx = {}
         self.requested_histories = {}
         self.requested_addrs = set()
         self.lock = Lock()
@@ -65,7 +68,7 @@ class Synchronizer(ThreadJob):
                 and not self.requested_addrs)
 
     def release(self):
-        self.network.unsubscribe(self.addr_subscription_response)
+        self.network.unsubscribe(self.on_address_status)
 
     def add(self, address):
         '''This can be called from the proxy or GUI threads.'''
@@ -75,9 +78,7 @@ class Synchronizer(ThreadJob):
     def subscribe_to_addresses(self, addresses):
         if addresses:
             self.requested_addrs |= addresses
-            msgs = map(lambda addr: ('blockchain.address.subscribe', [addr]),
-                       addresses)
-            self.network.send(msgs, self.addr_subscription_response)
+            self.network.subscribe_to_addresses(addresses, self.on_address_status)
 
     def get_status(self, h):
         if not h:
@@ -85,9 +86,9 @@ class Synchronizer(ThreadJob):
         status = ''
         for tx_hash, height in h:
             status += tx_hash + ':%d:' % height
-        return hashlib.sha256(status).digest().encode('hex')
+        return bh2u(hashlib.sha256(status.encode('ascii')).digest())
 
-    def addr_subscription_response(self, response):
+    def on_address_status(self, response):
         params, result = self.parse_response(response)
         if not params:
             return
@@ -96,13 +97,12 @@ class Synchronizer(ThreadJob):
         if self.get_status(history) != result:
             if self.requested_histories.get(addr) is None:
                 self.requested_histories[addr] = result
-                self.network.send([('blockchain.address.get_history', [addr])],
-                                  self.addr_history_response)
+                self.network.request_address_history(addr, self.on_address_history)
         # remove addr from list only after it is added to requested_histories
         if addr in self.requested_addrs:  # Notifications won't be in
             self.requested_addrs.remove(addr)
 
-    def addr_history_response(self, response):
+    def on_address_history(self, response):
         params, result = self.parse_response(response)
         if not params:
             return
@@ -110,7 +110,7 @@ class Synchronizer(ThreadJob):
         self.print_error("receiving history", addr, len(result))
         server_status = self.requested_histories[addr]
         hashes = set(map(lambda item: item['tx_hash'], result))
-        hist = map(lambda item: (item['tx_hash'], item['height']), result)
+        hist = list(map(lambda item: (item['tx_hash'], item['height']), result))
         # tx_fees
         tx_fees = [(item['tx_hash'], item.get('fee')) for item in result]
         tx_fees = dict(filter(lambda x:x[1] is not None, tx_fees))
@@ -135,16 +135,16 @@ class Synchronizer(ThreadJob):
         params, result = self.parse_response(response)
         if not params:
             return
-        tx_hash, tx_height = params
-        #assert tx_hash == hash_encode(Hash(result.decode('hex')))
+        tx_hash = params[0]
+        #assert tx_hash == hash_encode(Hash(bytes.fromhex(result)))
         tx = Transaction(result)
         try:
             tx.deserialize()
         except Exception:
             self.print_msg("cannot deserialize transaction, skipping", tx_hash)
             return
+        tx_height = self.requested_tx.pop(tx_hash)
         self.wallet.receive_tx_callback(tx_hash, tx, tx_height)
-        self.requested_tx.remove((tx_hash, tx_height))
         self.print_error("received tx %s height: %d bytes: %d" %
                          (tx_hash, tx_height, len(tx.raw)))
         # callbacks
@@ -155,15 +155,16 @@ class Synchronizer(ThreadJob):
 
     def request_missing_txs(self, hist):
         # "hist" is a list of [tx_hash, tx_height] lists
-        missing = set()
+        requests = []
         for tx_hash, tx_height in hist:
-            if self.wallet.transactions.get(tx_hash) is None:
-                missing.add((tx_hash, tx_height))
-        missing -= self.requested_tx
-        if missing:
-            requests = [('blockchain.transaction.get', tx) for tx in missing]
-            self.network.send(requests, self.tx_response)
-            self.requested_tx |= missing
+            if tx_hash in self.requested_tx:
+                continue
+            if tx_hash in self.wallet.transactions:
+                continue
+            requests.append(('blockchain.transaction.get', [tx_hash]))
+            self.requested_tx[tx_hash] = tx_height
+        self.network.send(requests, self.tx_response)
+
 
     def initialize(self):
         '''Check the initial state of the wallet.  Subscribe to all its
