@@ -1,4 +1,4 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 #
 # Electrum - lightweight Bitcoin client
 # Copyright (C) 2014 Thomas Voegtlin
@@ -26,12 +26,12 @@
 import re
 import platform
 from decimal import Decimal
-from urllib import quote
 
-from PyQt4.QtGui import *
-from PyQt4.QtCore import *
-import PyQt4.QtCore as QtCore
-import PyQt4.QtGui as QtGui
+from PyQt5.QtGui import *
+from PyQt5.QtCore import *
+import PyQt5.QtCore as QtCore
+import PyQt5.QtGui as QtGui
+from PyQt5.QtWidgets import (QHBoxLayout, QVBoxLayout, QLabel, QWidget)
 
 from electrum_gui.qt.qrcodewidget import QRCodeWidget
 from electrum.i18n import _
