@@ -1,4 +1,4 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 #
 # Electrum - lightweight Bitcoin client
 # Copyright (C) 2013 ecdsa@github
@@ -23,12 +23,12 @@
 # CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
 
-from PyQt4.QtGui import *
-from PyQt4.QtCore import *
+from PyQt5.QtGui import *
+from PyQt5.QtCore import *
 from electrum.i18n import _
 
-from util import *
-from qrtextedit import ShowQRTextEdit, ScanQRTextEdit
+from .util import *
+from .qrtextedit import ShowQRTextEdit, ScanQRTextEdit
 
 
 def seed_warning_msg(seed):
@@ -47,7 +47,6 @@ def seed_warning_msg(seed):
     ]) % len(seed.split())
 
 
-
 class SeedLayout(QVBoxLayout):
     #options
     is_bip39 = False
@@ -62,22 +61,20 @@ class SeedLayout(QVBoxLayout):
             vbox.addWidget(cb_ext)
         if 'bip39' in self.options:
             def f(b):
-                if b:
-                    msg = ' '.join([
-                        '<b>' + _('Warning') + '</b>' + ': ',
-                        _('BIP39 seeds may not be supported in the future.'),
-                        '<br/><br/>',
-                        _('As technology matures, Bitcoin address generation may change.'),
-                        _('However, BIP39 seeds do not include a version number.'),
-                        _('As a result, it is not possible to infer your wallet type from a BIP39 seed.'),
-                        '<br/><br/>',
-                        _('We do not guarantee that BIP39 seeds will be supported in future versions of Electrum.'),
-                        _('We recommend to use seeds generated by Electrum or compatible wallets.'),
-                    ])
-                    #self.parent.show_warning(msg)
-                self.seed_type_label.setVisible(not b)
                 self.is_seed = (lambda x: bool(x)) if b else self.saved_is_seed
                 self.on_edit()
+                self.is_bip39 = b
+                if b:
+                    msg = ' '.join([
+                        '<b>' + _('Warning') + ':</b>  ',
+                        _('BIP39 seeds can be imported in Electrum, so that users can access funds locked in other wallets.'),
+                        _('However, we do not generate BIP39 seeds, because they do not meet our safety standard.'),
+                        _('BIP39 seeds do not include a version number, which compromises compatibility with future software.'),
+                        _('We do not guarantee that BIP39 imports will always be supported in Electrum.'),
+                    ])
+                else:
+                    msg = ''
+                self.seed_warning.setText(msg)
             cb_bip39 = QCheckBox(_('BIP39 seed'))
             cb_bip39.toggled.connect(f)
             cb_bip39.setChecked(self.is_bip39)
@@ -88,7 +85,6 @@ class SeedLayout(QVBoxLayout):
         self.is_ext = cb_ext.isChecked() if 'ext' in self.options else False
         self.is_bip39 = cb_bip39.isChecked() if 'bip39' in self.options else False
 
-
     def __init__(self, seed=None, title=None, icon=True, msg=None, options=None, is_seed=None, passphrase=None, parent=None):
         QVBoxLayout.__init__(self)
         self.parent = parent
@@ -130,25 +126,31 @@ class SeedLayout(QVBoxLayout):
             hbox.addWidget(passphrase_e)
             self.addLayout(hbox)
         self.addStretch(1)
+        self.seed_warning = WWLabel('')
         if msg:
-            msg = seed_warning_msg(seed)
-            self.addWidget(WWLabel(msg))
+            self.seed_warning.setText(seed_warning_msg(seed))
+        self.addWidget(self.seed_warning)
 
     def get_seed(self):
-        text = unicode(self.seed_e.text())
+        text = self.seed_e.text()
         return ' '.join(text.split())
 
     def on_edit(self):
         from electrum.bitcoin import seed_type
         s = self.get_seed()
         b = self.is_seed(s)
-        t = seed_type(s)
-        label = _('Seed Type') + ': ' + t if t else ''
+        if not self.is_bip39:
+            t = seed_type(s)
+            label = _('Seed Type') + ': ' + t if t else ''
+        else:
+            from electrum.keystore import bip39_is_checksum_valid
+            is_checksum, is_wordlist = bip39_is_checksum_valid(s)
+            status = ('checksum: ' + ('ok' if is_checksum else 'failed')) if is_wordlist else 'unknown wordlist'
+            label = 'BIP39' + ' (%s)'%status
         self.seed_type_label.setText(label)
         self.parent.next_button.setEnabled(b)
 
 
-
 class KeysLayout(QVBoxLayout):
     def __init__(self, parent=None, title=None, is_valid=None):
         QVBoxLayout.__init__(self)
@@ -160,7 +162,7 @@ class KeysLayout(QVBoxLayout):
         self.addWidget(self.text_e)
 
     def get_text(self):
-        return unicode(self.text_e.text())
+        return self.text_e.text()
 
     def on_edit(self):
         b = self.is_valid(self.get_text())
